#include <algorithm>
#include <format>
#include <iostream>
#include <map>
#include <numeric>
#include <print>
#include <ranges>
#include <stack>
#include <string>
#include <string_view>
#include <vector>

class Interpreter {
  std::reference_wrapper<std::vector<Interpreter>> _interpreters;

  std::stack<int> _stack;
  std::stack<std::size_t> _reps;
  std::stack<std::size_t> _jmps;
  std::size_t _ic = 0;
  bool _finished = false;
  double _weight = 0.0;
  std::string_view _code;

  std::vector<int> _memory;

public:
  Interpreter(std::vector<Interpreter> &interpreters, std::string_view code)
      : _interpreters{interpreters}, _code{code}, _memory(0xffffff) {}

  int pop() {
    if (_stack.empty()) {
      return 0;
    }
    auto const value = _stack.top();
    _stack.pop();
    return value;
  }

  bool step() {
    if (_ic >= _code.length()) {
      return false;
    }

    char const ch = _code[_ic++];
    switch (ch) {
    case '#':
      _stack.push(0);
      break;
    case '+':
      ++_stack.top();
      break;
    case '@': {
      auto const address = pop();
      _stack.push(_memory[address]);
      break;
    }
    case '!': {
      auto const address = pop();
      auto const value = pop();
      _memory[address] = value;
      break;
    }
    case '?': {
      auto const value = pop();
      _interpreters.get().append_range(make_copies(value));
      break;
    }
    case '[': {
      auto const value = _stack.top();
      _reps.push(value);
      _jmps.push(_ic);
      break;
    }
    case ']':
      --_reps.top();
      _ic = _jmps.top();
      break;
    case '|': {
      auto const value = _stack.top();
      if (value == 0 or _reps.top() == 0) {
        _reps.pop();
        _jmps.pop();
        int braces = 1;

        while (true) {
          if (_code[_ic] == '[') {
            ++braces;
          } else if (_code[_ic] == ']') {
            if (braces == 0) {
              break;
            } else {
              --braces;
            }
          }
          ++_ic;
        }
        ++_ic;
      }
    }
    default:
      throw std::runtime_error(std::format("Invalid character {}", ch));
    }
    return true;
  }

  std::vector<Interpreter> make_copies(int n) {
    return to<std::vector>(std::ranges::views::iota(n) |
                           std::ranges::views::transform([&](auto i) {
                             auto copy = *this;
                             copy._weight = _weight / (n + 1);
                             copy._stack.push(_ic);
                             return copy;
                           }));
  }

  double weight() { return _weight; }
};

int interpret(std::string_view code) {
  std::vector<Interpreter> interpreters;
  std::vector<Interpreter> finished_interpreters;

  interpreters.emplace_back(interpreters, code);

  while (not interpreters.empty()) {
    for (int i = 0; i < interpreters.size();) {
      if (not interpreters[i].step()) {
        finished_interpreters.push_back(interpreters[i]);
        interpreters.erase(interpreters.begin() + i);
      } else {
        ++i;
      }
    }
  }

  std::map<int, double> outputs;
  for (auto &&interpreter : finished_interpreters) {
    outputs[interpreter.pop()] += interpreter.weight();
  }

  return std::ranges::max_element(
             outputs, [](auto &p1, auto &p2) { return p1.second < p2.second; })
      ->first;
}

int main() {
  std::string line;
  std::getline(std::cin, line);
  std::println("{"
               "}",
               interpret(line));
}